"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Diagnose = void 0;
const bc_minecraft_bedrock_project_1 = require("bc-minecraft-bedrock-project");
const main_1 = require("../../../../main");
const Json_1 = require("../../Json/Json");
const diagnostics_1 = require("../../Molang/diagnostics");
const commands_1 = require("../Mcfunction/commands");
/**Diagnoses the given document as an animation
 * @param doc The text document to diagnose
 * @param diagnoser The diagnoser builder to receive the errors*/
function Diagnose(doc, diagnoser) {
    //Check molang
    const text = doc.getText();
    (0, diagnostics_1.diagnose_molang)(text, "animation", diagnoser);
    const anims = Json_1.Json.LoadReport(doc, diagnoser);
    if (!bc_minecraft_bedrock_project_1.Internal.BehaviorPack.Animations.is(anims))
        return;
    //foreach animation, 
    bc_minecraft_bedrock_project_1.Map.forEach(anims.animations, (anim, id) => {
        const length = anim.animation_length;
        //foreach time
        bc_minecraft_bedrock_project_1.Map.forEach(anim.timeline, (data, time) => {
            (0, commands_1.json_commandscheck)(data, doc, diagnoser);
            if (length) {
                const temp = Number.parseFloat(time);
                if (temp > length)
                    diagnoser.Add(`${id}/timeline/${time}`, "Time specification is outside the animation range", main_1.DiagnosticSeverity.warning, "animation.time.range");
            }
        });
    });
}
exports.Diagnose = Diagnose;
//# sourceMappingURL=entry.js.map