"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.BehaviorPack = void 0;
const include_1 = require("bc-minecraft-bedrock-project/lib/src/Lib/Project/BehaviorPack/include");
const Animation = require("./Animation/entry");
const AnimationController = require("./Animation Controllers/entry");
const Block = require("./Block/entry");
const Entity = require("./Entity/entry");
const Mcfunction = require("./Mcfunction/entry");
const Tick = require("./Functions/Tick/entry");
const Item = require("./Item/entry");
const LootTable = require("./Loot Table/entry");
const Manifest = require("./Manifest/entry");
const Script = require("./Script/entry");
const SpawnRule = require("./Spawn Rule/entry");
const Structure = require("./Structure/entry");
const Trading = require("./Trading/entry");
var BehaviorPack;
(function (BehaviorPack) {
    /**Processes and diagnoses the given textdocument
     * @param doc The document to process / diagnose
     * @param diagnoser The diagnoser to report to
     * @returns `true` or `false` whenever or not it was succesfull*/
    function Process(doc, diagnoser) {
        //retrieve filter doc type
        const Type = include_1.FileType.detect(doc.uri);
        switch (Type) {
            case include_1.FileType.animation:
                Animation.Diagnose(doc, diagnoser);
                break;
            case include_1.FileType.animation_controller:
                AnimationController.Diagnose(doc, diagnoser);
                break;
            case include_1.FileType.block:
                Block.Diagnose(doc, diagnoser);
                break;
            case include_1.FileType.entity:
                Entity.Diagnose(doc, diagnoser);
                break;
            case include_1.FileType.function:
                if (doc.uri.endsWith("tick.json")) {
                    Tick.Diagnose(doc, diagnoser);
                }
                else {
                    Mcfunction.Diagnose(doc, diagnoser);
                }
                break;
            case include_1.FileType.item:
                Item.Diagnose(doc, diagnoser);
                break;
            case include_1.FileType.loot_table:
                LootTable.Diagnose(doc, diagnoser);
                break;
            case include_1.FileType.manifest:
                Manifest.Diagnose(doc, diagnoser);
                break;
            case include_1.FileType.script:
                Script.Diagnose(doc, diagnoser);
                break;
            case include_1.FileType.spawn_rule:
                SpawnRule.Diagnose(doc, diagnoser);
                break;
            case include_1.FileType.structure:
                Structure.Diagnose(doc, diagnoser);
                break;
            case include_1.FileType.trading:
                Trading.Diagnose(doc, diagnoser);
                break;
            default:
            case include_1.FileType.unknown:
                return false;
        }
        return true;
    }
    BehaviorPack.Process = Process;
})(BehaviorPack = exports.BehaviorPack || (exports.BehaviorPack = {}));
//# sourceMappingURL=BehaviorPack.js.map