"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.behaviorpack_check_blockstates = exports.behaviorpack_check_blockid = exports.behaviorpack_check_blockdescriptor = void 0;
const bc_minecraft_bedrock_types_1 = require("bc-minecraft-bedrock-types");
const bc_minecraft_bedrock_vanilla_data_1 = require("bc-minecraft-bedrock-vanilla-data");
const Severity_1 = require("../../../Types/DiagnosticsBuilder/Severity");
const Definitions_1 = require("../../Definitions");
/**
 *
 * @param blockDescriptor
 * @param diagnoser
 */
function behaviorpack_check_blockdescriptor(blockDescriptor, diagnoser) {
    behaviorpack_check_blockid(blockDescriptor, diagnoser);
    behaviorpack_check_blockstates(blockDescriptor, diagnoser);
}
exports.behaviorpack_check_blockdescriptor = behaviorpack_check_blockdescriptor;
/**Checks if the blocks exists in the project or in vanilla, if not then a bug is reported
 * @param id
 * @param diagnoser
 * @returns
 */
function behaviorpack_check_blockid(blockDescriptor, diagnoser) {
    let id = bc_minecraft_bedrock_types_1.Minecraft.Block.getId(blockDescriptor.text);
    //Defined in McProject
    if ((0, Definitions_1.check_definition_value)(diagnoser.project.definitions.block, id, diagnoser))
        return;
    const data = diagnoser.context.getCache();
    //Project has block
    if (data.hasBlock(id))
        return;
    const edu = (0, Definitions_1.education_enabled)(diagnoser);
    //Vanilla has block
    if (bc_minecraft_bedrock_vanilla_data_1.MinecraftData.BehaviorPack.hasBlock(id, edu))
        return;
    //Missing namespace?
    if (!id.includes(":")) {
        //retry
        id = "minecraft:" + id;
        //Defined in McProject
        if ((0, Definitions_1.check_definition_value)(diagnoser.project.definitions.block, id, diagnoser))
            return;
        //Project has block
        if (data.hasBlock(id))
            return;
        //Vanilla has block
        if (bc_minecraft_bedrock_vanilla_data_1.MinecraftData.BehaviorPack.hasBlock(id, edu))
            return;
    }
    //Nothing then report error
    diagnoser.Add(id, `Cannot find block definition: ${id}`, Severity_1.DiagnosticSeverity.error, "behaviorpack.block.missing");
}
exports.behaviorpack_check_blockid = behaviorpack_check_blockid;
/**Checks if the blocks exists in the project or in vanilla, if not then a bug is reported
 * @param id
 * @param diagnoser
 * @returns
 */
function behaviorpack_check_blockstates(blockDescriptor, diagnoser) {
    //If the block has no states then skip
    if (!blockDescriptor.text.includes("["))
        return;
    //Parses states
    const blockData = bc_minecraft_bedrock_types_1.Minecraft.Block.fromBlockDescriptor(blockDescriptor.text);
    const data = diagnoser.context.getCache();
    const block = data.BehaviorPacks.blocks.get(blockData.id);
    //No block found, expecting behaviorpack_check_blockid has been run
    if (!block)
        return;
    if (block.states.length == 0 && blockData.states.length > 0) {
        //Block has not defined states, but states are being used
        diagnoser.Add(blockDescriptor, `Block: ${block.id} has no defined states, but the block descriptor does: '${blockDescriptor}'`, Severity_1.DiagnosticSeverity.error, "behaviorpack.block.states.missing");
        return;
    }
    for (var I = 0; I < blockData.states.length; I++) {
        const state = blockData.states[I];
        check_state(blockDescriptor, state, block, diagnoser);
    }
}
exports.behaviorpack_check_blockstates = behaviorpack_check_blockstates;
/**
 *
 * @param blockDescriptor
 * @param state
 * @param data
 * @param diagnoser
 * @returns
 */
function check_state(blockDescriptor, state, data, diagnoser) {
    for (var I = 0; I < data.states.length; I++) {
        const stateData = data.states[I];
        //If found state with the same name
        if (stateData.name === state.property) {
            if (!stateData.values.includes(state.value)) {
            }
            return;
        }
    }
    //No state matching found
    diagnoser.Add(blockDescriptor, `Missing state: '${state.property}' in the block definition: '${data.id}'`, Severity_1.DiagnosticSeverity.error, "behaviorpack.block.states.missing");
}
//# sourceMappingURL=diagnose.js.map