"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.behaviorpack_entity_event_diagnose = exports.behaviorpack_entity_spawnegg_diagnose = exports.behaviorpack_entityid_diagnose = void 0;
const bc_minecraft_bedrock_command_1 = require("bc-minecraft-bedrock-command");
const bc_minecraft_bedrock_vanilla_data_1 = require("bc-minecraft-bedrock-vanilla-data");
const Severity_1 = require("../../../Types/DiagnosticsBuilder/Severity");
const Definitions_1 = require("../../Definitions");
/**Checks if the entities exists in the project or in vanilla, if not then a bug is reported
 * @param id
 * @param diagnoser
 * @returns
 */
function behaviorpack_entityid_diagnose(value, diagnoser) {
    let id = value.text;
    //Defined in McProject
    if ((0, Definitions_1.check_definition_value)(diagnoser.project.definitions.entity, id, diagnoser))
        return;
    const data = diagnoser.context.getCache();
    //Project has entity
    if (data.hasEntity(id))
        return;
    const edu = (0, Definitions_1.education_enabled)(diagnoser);
    //Vanilla has entity
    if (bc_minecraft_bedrock_vanilla_data_1.MinecraftData.BehaviorPack.hasEntity(id, edu))
        return;
    //No namespace?
    if (!id.includes(':')) {
        id = "minecraft:" + id;
        //Defined in McProject
        if ((0, Definitions_1.check_definition_value)(diagnoser.project.definitions.entity, id, diagnoser))
            return;
        const data = diagnoser.context.getCache();
        //Project has entity
        if (data.hasEntity(id))
            return;
        const edu = (0, Definitions_1.education_enabled)(diagnoser);
        //Vanilla has entity
        if (bc_minecraft_bedrock_vanilla_data_1.MinecraftData.BehaviorPack.hasEntity(id, edu))
            return;
    }
    //Nothing then report error
    diagnoser.Add(value.offset, `Cannot find entity definition: ${id}`, Severity_1.DiagnosticSeverity.error, "behaviorpack.entity.missing");
}
exports.behaviorpack_entityid_diagnose = behaviorpack_entityid_diagnose;
/**Checks if the entities exists in the project or in vanilla, if not then a bug is reported
 * @param id
 * @param diagnoser
 * @returns
 */
function behaviorpack_entity_spawnegg_diagnose(value, diagnoser) {
    const id = value.text.replace('_spawn_egg', "");
    behaviorpack_entityid_diagnose({ offset: value.offset, text: id }, diagnoser);
}
exports.behaviorpack_entity_spawnegg_diagnose = behaviorpack_entity_spawnegg_diagnose;
/**Checks if the event is defined on the correct entities
 * @param data
 * @param builder
 * @param Com
 */
function behaviorpack_entity_event_diagnose(data, diagnoser, Com) {
    var _a;
    const edu = (0, Definitions_1.education_enabled)(diagnoser);
    const matches = Com.getBestMatch(edu);
    if (matches.length !== 1)
        return;
    const entityid_index = matches[0].parameters.findIndex(p => { p.type === bc_minecraft_bedrock_command_1.ParameterType.entity; });
    let entityid = undefined;
    if (entityid_index >= 0) {
        entityid = (_a = Com.parameters[entityid_index]) === null || _a === void 0 ? void 0 : _a.text;
    }
    else {
        //TODO selector parsing?
    }
    if (entityid) {
        //Get entity
        const entity = diagnoser.context.getCache().BehaviorPacks.entities.get(entityid);
        //Entity found
        if (entity) {
            //Events not found
            if (!entity.events.includes(data.text)) {
                diagnoser.Add(data.offset, `Entity: ${entityid} has no event declared: ${data.text}`, Severity_1.DiagnosticSeverity.error, "behaviorpack.entity.event.missing");
            }
        }
    }
}
exports.behaviorpack_entity_event_diagnose = behaviorpack_entity_event_diagnose;
//# sourceMappingURL=diagnose.js.map