"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Diagnose = void 0;
const bc_minecraft_bedrock_project_1 = require("bc-minecraft-bedrock-project");
const Json_1 = require("../../Json/Json");
const components_1 = require("./components");
const anim_or_controller_1 = require("../anim or controller");
const bc_minecraft_molang_1 = require("bc-minecraft-molang");
const bc_minecraft_bedrock_types_1 = require("bc-minecraft-bedrock-types");
const diagnostics_1 = require("../../Molang/diagnostics");
const Script_1 = require("../../Minecraft/Script");
const events_1 = require("./events");
/**Diagnoses the given document as an bp entity
 * @param doc The text document to diagnose
 * @param diagnoser The diagnoser builder to receive the errors*/
function Diagnose(doc, diagnoser) {
    //Check molang
    (0, diagnostics_1.diagnose_molang)(doc.getText(), "entity", diagnoser);
    const entity = Json_1.Json.LoadReport(doc, diagnoser);
    if (!bc_minecraft_bedrock_project_1.Internal.BehaviorPack.Entity.is(entity))
        return;
    //No resourcepack check, entities can exist without their rp side
    //check components
    (0, components_1.behaviorpack_entity_components_dependencies)(entity, diagnoser);
    const container = entity["minecraft:entity"];
    const MolangData = bc_minecraft_molang_1.Molang.MolangFullSet.harvest(container);
    const id = container.description.identifier;
    const owner = {
        id: id,
        molang: MolangData,
        animations: bc_minecraft_molang_1.DefinedUsing.create(),
    };
    //Convert animations / controllers
    bc_minecraft_bedrock_types_1.Types.Definition.forEach(container.description.animations, (ref, anim_id) => {
        owner.animations.defined.push(ref);
        owner.animations.using.push(anim_id);
    });
    //Check animations / animation controllers implements
    owner.animations.using.forEach((anim_id) => (0, anim_or_controller_1.animation_or_controller_diagnose_implementation)(anim_id, owner, "entity", diagnoser));
    //Script check
    if (container.description.scripts)
        (0, Script_1.diagnose_script)(diagnoser, container.description.scripts, container.description.animations);
    //Check events
    if (container.events)
        (0, events_1.behaviorpack_entity_check_events)(container.events, diagnoser, container.component_groups);
}
exports.Diagnose = Diagnose;
//# sourceMappingURL=entry.js.map