"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.commandscheck = exports.json_commandscheck = exports.mcfunction_commandscheck = void 0;
const bc_minecraft_bedrock_command_1 = require("bc-minecraft-bedrock-command");
const include_1 = require("../../../Types/DiagnosticsBuilder/include");
const Definitions_1 = require("../../Definitions");
const Boolean_1 = require("../../General/Boolean");
const Float_1 = require("../../General/Float");
const Integer_1 = require("../../General/Integer");
const String_1 = require("../../General/String");
const Commands_1 = require("../../Minecraft/Commands");
const Coordinate_1 = require("../../Minecraft/Coordinate");
const Effect_1 = require("../../Minecraft/Effect");
const JsonItem_1 = require("../../Minecraft/JsonItem");
const JsonRawText_1 = require("../../Minecraft/JsonRawText");
const Objective_1 = require("../../Minecraft/Objective");
const Selector_1 = require("../../Minecraft/Selector");
const Tag_1 = require("../../Minecraft/Tag");
const diagnose_1 = require("../Structure/diagnose");
const diagnose_2 = require("../Block/diagnose");
const diagnose_3 = require("../Entity/diagnose");
const diagnose_4 = require("./diagnose");
const diagnose_5 = require("../../Mode/diagnose");
const diagnose_6 = require("../Item/diagnose");
const Xp_1 = require("../../Minecraft/Xp");
const Keyword_1 = require("../../General/Keyword");
const Tickingarea_1 = require("../../Minecraft/Tickingarea");
const diagnose_7 = require("../../ResourcePack/Particle/diagnose");
const diagnose_8 = require("../../ResourcePack/Sounds Definitions/diagnose");
const anim_or_controller_1 = require("../../ResourcePack/anim or controller");
/**
 *
 * @param doc
 * @param diagnoser
 */
function mcfunction_commandscheck(doc, diagnoser) {
    const edu = (0, Definitions_1.education_enabled)(diagnoser);
    const text = doc.getText();
    const lines = text.split("\n");
    for (let I = 0; I < lines.length; I++) {
        const line = lines[I].trim();
        if (line === "")
            continue;
        //If the line is a whole comment then skip
        if (line.startsWith("#"))
            continue;
        const offset = text.indexOf(line);
        let comm = bc_minecraft_bedrock_command_1.Command.parse(line, offset);
        if (comm.isEmpty())
            continue;
        while (comm) {
            mcfunction_commandcheck(comm, diagnoser, edu);
            comm = comm.getSubCommand();
        }
    }
}
exports.mcfunction_commandscheck = mcfunction_commandscheck;
/**
 *
 * @param prop
 * @param doc
 * @param diagnoser
 */
function json_commandscheck(prop, doc, diagnoser) {
    if (typeof prop === "string") {
        if (prop.startsWith("/")) {
            commandscheck(prop.substring(1), doc, diagnoser);
        }
    }
    else {
        prop.forEach((p) => {
            if (p.startsWith("/")) {
                commandscheck(p.substring(1), doc, diagnoser);
            }
        });
    }
}
exports.json_commandscheck = json_commandscheck;
/**
 *
 * @param commandtext
 * @param doc
 * @param diagnoser
 * @returns
 */
function commandscheck(commandtext, doc, diagnoser) {
    if (commandtext.length < 3)
        return;
    const edu = (0, Definitions_1.education_enabled)(diagnoser);
    const offset = doc.getText().indexOf(commandtext);
    let comm = bc_minecraft_bedrock_command_1.Command.parse(commandtext, offset);
    if (comm.isEmpty())
        return;
    while (comm) {
        mcfunction_commandcheck(comm, diagnoser, edu);
        comm = comm.getSubCommand();
    }
}
exports.commandscheck = commandscheck;
/**
 *
 * @param command
 * @param diagnoser
 * @param edu
 * @returns
 */
function mcfunction_commandcheck(command, diagnoser, edu) {
    const info = command.getBestMatch(edu);
    if (info.length === 0) {
        diagnoser.Add(command.parameters[0].offset, `Unknown command syntax: "${command.getKeyword()}"`, include_1.DiagnosticSeverity.error, "mcfunction.syntax.unknown");
        return;
    }
    const data = info[0];
    const max = Math.min(data.parameters.length, command.parameters.length);
    for (let I = 0; I < max; I++) {
        mcfunction_diagnoseparameter(data.parameters[I], command.parameters[I], diagnoser, command, edu);
    }
}
/**Switch data*/
const ParameterDiagnostics = {
    [bc_minecraft_bedrock_command_1.ParameterType.animation]: anim_or_controller_1.animation_reference_diagnose,
    [bc_minecraft_bedrock_command_1.ParameterType.block]: diagnose_2.behaviorpack_check_blockdescriptor,
    [bc_minecraft_bedrock_command_1.ParameterType.blockStates]: diagnose_2.behaviorpack_check_blockstates,
    [bc_minecraft_bedrock_command_1.ParameterType.boolean]: Boolean_1.general_boolean_diagnose,
    [bc_minecraft_bedrock_command_1.ParameterType.cameraShakeType]: diagnose_5.mode_camerashake_diagnose,
    [bc_minecraft_bedrock_command_1.ParameterType.cloneMode]: diagnose_5.mode_clone_diagnose,
    //Custom call [ParameterType.command]:,
    [bc_minecraft_bedrock_command_1.ParameterType.coordinate]: Coordinate_1.minecraft_coordinate_diagnose,
    [bc_minecraft_bedrock_command_1.ParameterType.difficulty]: diagnose_5.mode_difficulty_diagnose,
    [bc_minecraft_bedrock_command_1.ParameterType.effect]: Effect_1.minecraft_effect_diagnose,
    [bc_minecraft_bedrock_command_1.ParameterType.entity]: diagnose_3.behaviorpack_entityid_diagnose,
    //Custom call [ParameterType.event]:behaviorpack_entity_event_diagnose,
    [bc_minecraft_bedrock_command_1.ParameterType.fillMode]: diagnose_5.mode_fill_diagnose,
    [bc_minecraft_bedrock_command_1.ParameterType.function]: diagnose_4.behaviorpack_functions_diagnose,
    [bc_minecraft_bedrock_command_1.ParameterType.float]: Float_1.general_float_diagnose,
    [bc_minecraft_bedrock_command_1.ParameterType.gamemode]: diagnose_5.mode_gamemode_diagnose,
    [bc_minecraft_bedrock_command_1.ParameterType.integer]: Integer_1.general_integer_diagnose,
    [bc_minecraft_bedrock_command_1.ParameterType.item]: (item, diagnoser) => {
        if (item.text.endsWith("_spawn_egg")) {
            (0, diagnose_3.behaviorpack_entity_spawnegg_diagnose)(item, diagnoser);
        }
        else {
            (0, diagnose_6.behaviorpack_item_diagnose)(item, diagnoser);
        }
    },
    [bc_minecraft_bedrock_command_1.ParameterType.jsonItem]: JsonItem_1.minecraft_jsonitem_diagnose,
    [bc_minecraft_bedrock_command_1.ParameterType.jsonRawText]: JsonRawText_1.minecraft_jsonrawtext_diagnose,
    //Custom call [ParameterType.keyword]:general_keyword_diagnose,
    [bc_minecraft_bedrock_command_1.ParameterType.locateFeature]: diagnose_5.mode_locatefeature_diagnose,
    //Custom call [ParameterType.message]:,
    [bc_minecraft_bedrock_command_1.ParameterType.maskMode]: diagnose_5.mode_mask_diagnose,
    [bc_minecraft_bedrock_command_1.ParameterType.mirror]: diagnose_5.mode_mirror_diagnose,
    [bc_minecraft_bedrock_command_1.ParameterType.musicRepeatMode]: diagnose_5.mode_musicrepeat_diagnose,
    [bc_minecraft_bedrock_command_1.ParameterType.objective]: Objective_1.minecraft_objectives_diagnose,
    [bc_minecraft_bedrock_command_1.ParameterType.oldBlockMode]: diagnose_5.mode_oldblock_diagnose,
    [bc_minecraft_bedrock_command_1.ParameterType.operation]: diagnose_5.mode_operation_diagnose,
    [bc_minecraft_bedrock_command_1.ParameterType.particle]: diagnose_7.resourcepack_particle_diagnose,
    [bc_minecraft_bedrock_command_1.ParameterType.replaceMode]: diagnose_5.mode_replace_diagnose,
    [bc_minecraft_bedrock_command_1.ParameterType.rideRules]: diagnose_5.mode_riderules_diagnose,
    [bc_minecraft_bedrock_command_1.ParameterType.ridefillMode]: diagnose_5.mode_ridefill_diagnose,
    [bc_minecraft_bedrock_command_1.ParameterType.rotation]: diagnose_5.mode_rotation_diagnose,
    [bc_minecraft_bedrock_command_1.ParameterType.saveMode]: diagnose_5.mode_save_diagnose,
    //Custom call [ParameterType.selector]:minecraft_selector_diagnose,
    [bc_minecraft_bedrock_command_1.ParameterType.slotType]: diagnose_5.mode_slottype_diagnose,
    //Custom call [ParameterType.slotID]:,
    [bc_minecraft_bedrock_command_1.ParameterType.sound]: diagnose_8.resourcepack_sound_diagnose,
    [bc_minecraft_bedrock_command_1.ParameterType.string]: String_1.general_string_diagnose,
    [bc_minecraft_bedrock_command_1.ParameterType.structure]: diagnose_1.behaviorpack_structure_diagnose,
    [bc_minecraft_bedrock_command_1.ParameterType.structureAnimationMode]: diagnose_5.mode_structureanimation_diagnose,
    [bc_minecraft_bedrock_command_1.ParameterType.tag]: Tag_1.minecraft_tag_diagnose,
    [bc_minecraft_bedrock_command_1.ParameterType.teleportRules]: diagnose_5.mode_teleportrules_diagnose,
    [bc_minecraft_bedrock_command_1.ParameterType.tickingarea]: Tickingarea_1.minecraft_tickingarea_diagnose,
    [bc_minecraft_bedrock_command_1.ParameterType.time]: diagnose_5.mode_time_diagnose,
    //Custom call ParameterType.unknown]:(item, diagnoser)=>diagnoser.Add(item.offset, "Unknown parametype: " + item.type, DiagnosticSeverity.warning, "debugger.error"),
    [bc_minecraft_bedrock_command_1.ParameterType.xp]: Xp_1.minecraft_xp_diagnose,
};
/**
 *
 * @param pattern
 * @param data
 * @param diagnoser
 * @param Com
 * @param edu
 * @returns
 */
function mcfunction_diagnoseparameter(pattern, data, diagnoser, Com, edu) {
    var _a;
    if (pattern === undefined || data === undefined)
        return;
    if (pattern.options) {
        //If wildcard is allowed and the text is an wildcard, then skip diagnose
        if (pattern.options.wildcard === true) {
            if (data.text === "*")
                return;
        }
        //If accepted values is filled in and the text is a match, then skip diagnose
        if ((_a = pattern.options.acceptedValues) === null || _a === void 0 ? void 0 : _a.includes(data.text)) {
            return;
        }
    }
    //Get specific call
    const call = ParameterDiagnostics[pattern.type];
    //If call is found, then use that
    if (call) {
        return call(data, diagnoser);
    }
    //Custom calls
    switch (pattern.type) {
        case bc_minecraft_bedrock_command_1.ParameterType.command:
            return (0, Commands_1.minecraft_check_command)(data, diagnoser, edu);
        case bc_minecraft_bedrock_command_1.ParameterType.event:
            return (0, diagnose_3.behaviorpack_entity_event_diagnose)(data, diagnoser, Com);
        case bc_minecraft_bedrock_command_1.ParameterType.keyword:
            return (0, Keyword_1.general_keyword_diagnose)(pattern.text, data, diagnoser);
        case bc_minecraft_bedrock_command_1.ParameterType.message:
            return true; //TODO message check
        case bc_minecraft_bedrock_command_1.ParameterType.selector:
            return (0, Selector_1.minecraft_selector_diagnose)(pattern, data, diagnoser);
        case bc_minecraft_bedrock_command_1.ParameterType.slotID:
            return (0, diagnose_5.mode_slotid_diagnose)(data, Com, diagnoser);
        case bc_minecraft_bedrock_command_1.ParameterType.unknown:
            diagnoser.Add(data.offset, `Unknown parametype: ${pattern.type}:${bc_minecraft_bedrock_command_1.ParameterType[pattern.type]}`, include_1.DiagnosticSeverity.warning, "debugger.error");
            return false;
    }
}
//# sourceMappingURL=commands.js.map