"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.general_animation_controllers_implementation = exports.general_animation_controller = exports.general_animation_controllers = void 0;
const bc_minecraft_bedrock_project_1 = require("bc-minecraft-bedrock-project");
const Severity_1 = require("../../Types/DiagnosticsBuilder/Severity");
const diagnostics_1 = require("../Molang/diagnostics");
/**
 *
 * @param data
 * @param diagnoser
 */
function general_animation_controllers(data, diagnoser) {
    bc_minecraft_bedrock_project_1.Map.forEach(data.animation_controllers, (controller, controller_id) => {
        general_animation_controller(controller, controller_id, diagnoser);
    });
}
exports.general_animation_controllers = general_animation_controllers;
/**
 *
 * @param controller
 * @param controller_id
 * @param diagnoser
 */
function general_animation_controller(controller, controller_id, diagnoser) {
    //Check if initial_state points to existing state
    if (controller.initial_state) {
        const initial_state = controller.initial_state;
        if (controller.states[initial_state] === undefined) {
            diagnoser.Add(`${controller_id}/initial_state/${initial_state}`, "Cannot find initial state, minecraft will revert to state at 0 index", Severity_1.DiagnosticSeverity.warning, "animation_controller.state.missing");
        }
    }
    //Check states
    bc_minecraft_bedrock_project_1.Map.forEach(controller.states, (state, state_id) => {
        //Check transitions
        if (state.transitions)
            CheckTransition(controller_id, state.transitions, controller.states, diagnoser);
    });
}
exports.general_animation_controller = general_animation_controller;
/**
 *
 * @param controller
 * @param Transitions
 * @param States
 * @param Builder
 */
function CheckTransition(controller, Transitions, States, diagnoser) {
    //Loop over the transitions
    for (var I = 0; I < Transitions.length; I++) {
        const trans = Transitions[I];
        //Get state identification refered
        const state = typeof trans === "string" ? trans : Object.getOwnPropertyNames(trans)[0];
        //check is map contains any value
        if (States[state] === undefined) {
            diagnoser.Add(controller + "/states/" + state, `missing state defined by transition: ${state}`, Severity_1.DiagnosticSeverity.error, "animation_controller.state.missing");
        }
    }
}
function general_animation_controllers_implementation(controller, user, ownerType, diagnoser) {
    //for each animation
    controller.animations.using.forEach((anim_id) => {
        if (user.animations.defined.includes(anim_id))
            return;
        diagnoser.Add(`${user.id}/${controller.id}`, `Animation controller (${controller.id}) is using animation: '${anim_id}' but ${user.id} has nothing defined that matches the given key\nMinecraft will still run but might return null errors on the animation`, Severity_1.DiagnosticSeverity.warning, "minecraft.animation_controller.animation.undefined");
    });
    //Molang
    (0, diagnostics_1.diagnose_molang_implementation)(controller, user, ownerType, diagnoser);
}
exports.general_animation_controllers_implementation = general_animation_controllers_implementation;
//# sourceMappingURL=Animation%20Controllers.js.map