"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.minecraft_selector_diagnose = void 0;
const bc_minecraft_bedrock_project_1 = require("bc-minecraft-bedrock-project");
const bc_minecraft_bedrock_types_1 = require("bc-minecraft-bedrock-types");
const include_1 = require("../../../Lib/Types/DiagnosticsBuilder/include");
const bc_minecraft_bedrock_types_2 = require("bc-minecraft-bedrock-types");
const Definitions_1 = require("../Definitions");
const Range_1 = require("../General/Range");
const diagnose_1 = require("../Mode/diagnose");
const Coordinate_1 = require("./Coordinate");
const Name_1 = require("./Name");
const Objective_1 = require("./Objective");
const Family_1 = require("./Family");
const diagnose_2 = require("../BehaviorPack/Entity/diagnose");
const Tag_1 = require("./Tag");
const Float_1 = require("../General/Float");
const Integer_1 = require("../General/Integer");
function minecraft_selector_diagnose(pattern, value, diagnoser) {
    var _a, _b;
    const sel = value.text;
    //Is a selector?
    if (sel.startsWith("@")) {
        minecraft_selector_diagnose_hard(pattern, value, diagnoser);
        return;
    }
    //Fake entity or named then
    const name = bc_minecraft_bedrock_project_1.Text.UnQuote(sel);
    //Fake players have been banned
    if (((_a = pattern.options) === null || _a === void 0 ? void 0 : _a.allowFakePlayers) === false) {
        diagnoser.Add(value, "No fake players / names allowed", include_1.DiagnosticSeverity.error, "minecraft.selector.invalid");
        return;
    }
    if (((_b = pattern.options) === null || _b === void 0 ? void 0 : _b.playerOnly) === true) {
        diagnoser.Add(value, "Only players selector allowed to be used", include_1.DiagnosticSeverity.error, "minecraft.selector.invalid");
        return;
    }
    const data = diagnoser.context.getCache();
    //Defined in McProject
    if ((0, Definitions_1.check_definition_value)(diagnoser.project.definitions.name, name, diagnoser))
        return;
    //Project has defined this fake entity
    if (data.General.fakeEntities.has(name))
        return;
    //Found nothing then report
    diagnoser.Add(value, `Cannot find fake entity definition or name for: ${name}`, include_1.DiagnosticSeverity.warning, "minecraft.name.missing");
}
exports.minecraft_selector_diagnose = minecraft_selector_diagnose;
/**
 *
 * @param pattern
 * @param value
 * @param diagnoser
 */
function minecraft_selector_diagnose_hard(pattern, value, diagnoser) {
    var _a;
    const selector = bc_minecraft_bedrock_types_1.Minecraft.Selector.parse(value.text, value.offset);
    //If the selector is only meant to be aimed at player warn the user
    if (((_a = pattern.options) === null || _a === void 0 ? void 0 : _a.playerOnly) === true) {
        if (selector.type === "@e") {
            diagnoser.Add(value, "Selector is meant to target only players", include_1.DiagnosticSeverity.info, "minecraft.selector.playeronly");
        }
    }
    if (!bc_minecraft_bedrock_types_1.Minecraft.Selector.isValidType(selector)) {
        diagnoser.Add(value, `Unknown selector type: ${selector.type}`, include_1.DiagnosticSeverity.error, "minecraft.selector.type.invalid");
    }
    //Check for duplicate scores definition
    selector_scores_duplicate(value, diagnoser);
    //Check attributes
    selector.attributes.forEach((attr) => minecraft_selector_attribute_diagnose(attr, selector, diagnoser));
    //Scheck scores
    selector.scores.forEach((score) => minecraft_selector_score_attribute_diagnose(score, diagnoser));
}
function selector_scores_duplicate(value, diagnoser) {
    const firstIndex = value.text.indexOf("scores={");
    if (firstIndex < 0)
        return;
    const secondindex = value.text.indexOf("scores={", firstIndex + 5);
    if (secondindex < 0)
        return;
    diagnoser.Add(value, "Selector has multiple scores definitions. Only one is allowed", include_1.DiagnosticSeverity.error, "minecraft.selector.scores.duplicate");
}
/**
 *
 * @param attr
 * @param sel
 * @param diagnoser
 * @returns
 */
function minecraft_selector_attribute_diagnose(attr, sel, diagnoser) {
    //Attribute doesn't exist then skip it
    if (!(0, diagnose_1.mode_selectorattribute_diagnose)(attr, diagnoser))
        return;
    const word = attr.getValue();
    if (word.text.startsWith("!")) {
        word.text = word.text.slice(1);
    }
    const old = word.text;
    word.text = old.trim();
    word.offset += old.indexOf(word.text);
    switch (attr.name) {
        case "rx":
        case "rxm":
        case "ry":
        case "rym":
            selectorattribute_no_duplicates(attr, sel, diagnoser);
            (0, Float_1.general_float_diagnose)(word, diagnoser, { min: -180, max: 180 });
            return;
        case "x":
        case "y":
        case "z":
        case "dx":
        case "dy":
        case "dz":
            selectorattribute_no_duplicates(attr, sel, diagnoser);
            return selectorattribute_coordinate(word, diagnoser);
        case "r":
        case "rm":
            selectorattribute_no_duplicates(attr, sel, diagnoser);
            (0, Float_1.general_positive_float_diagnose)(word, diagnoser);
            return;
        case "c":
            selectorattribute_no_duplicates(attr, sel, diagnoser);
            (0, Integer_1.general_integer_diagnose)(word, diagnoser);
            return;
        case "l":
        case "lm":
            selectorattribute_no_duplicates(attr, sel, diagnoser);
            (0, Integer_1.general_positive_integer_diagnose)(word, diagnoser);
            return;
        case "m":
            //Types and gamemode can only be tested postive once, but can have all the negative tests
            (0, diagnose_1.mode_gamemode_diagnose)(word, diagnoser);
            return selectorattribute_postive_all_negatives(attr, sel, diagnoser);
        case "family":
            //Family attribute is allowed multiple tests
            (0, Family_1.minecraft_family_diagnose)(word, diagnoser);
            return selectorattribute_all(attr, sel, diagnoser);
        case "tag":
            //Family attribute is allowed multiple tests
            (0, Tag_1.minecraft_tag_diagnose)(word, diagnoser);
            return selectorattribute_all(attr, sel, diagnoser);
        case "type":
            //Types and gamemode can only be tested postive once, but can have all the negative tests
            (0, diagnose_2.behaviorpack_entityid_diagnose)(word, diagnoser);
            return selectorattribute_postive_all_negatives(attr, sel, diagnoser);
        case "name":
            selectorattribute_postive_all_negatives(attr, sel, diagnoser);
            return (0, Name_1.minecraft_name_diagnose)(word, diagnoser);
    }
}
/**
 *
 * @param attr
 * @param diagnoser
 */
function minecraft_selector_score_attribute_diagnose(attr, diagnoser) {
    const value_offset = attr.offset + attr.name.length + 1;
    const not = attr.value.startsWith("!");
    const value = not ? attr.value.substring(1) : attr.value;
    //Check objective references
    (0, Objective_1.minecraft_objectives_diagnose)(bc_minecraft_bedrock_types_2.Types.OffsetWord.create(attr.name, attr.offset), diagnoser);
    //Check range value
    (0, Range_1.general_range_integer_diagnose)(bc_minecraft_bedrock_types_2.Types.OffsetWord.create(value, value_offset), diagnoser);
}
/**
 *
 * @param name
 * @param selector
 * @param receiver
 */
function selectorattribute_coordinate(value, diagnoser) {
    if (value.text.startsWith("^"))
        diagnoser.Add(value, "Selector attribute coordinate cannot be local coordinates types, only relative or absolute", include_1.DiagnosticSeverity.error, "selector.coordinate.invalid");
    (0, Coordinate_1.minecraft_coordinate_diagnose)(value, diagnoser);
}
/**
 *
 * @param name
 * @param selector
 * @param receiver
 */
function selectorattribute_no_duplicates(name, selector, diagnoser) {
    var Count = selector.count(name.name);
    if (Count > 1) {
        diagnoser.Add(name.getName(), `Selector attribute: ${name} can only be used once in a selector`, include_1.DiagnosticSeverity.error, "selector.attribute.noduplicate");
    }
}
/**
 *
 * @param value
 * @param selector
 * @param diagnoser
 * @returns
 */
function selectorattribute_postive_all_negatives(value, selector, diagnoser) {
    //Attribute can only be tested postive once, but can have all the negative tests
    const parameters = selector.get(value.name);
    if (parameters.length <= 1)
        return;
    const name = value.name;
    const offset = value.offset;
    let Negatives = 0;
    for (let index = 0; index < parameters.length; index++) {
        const element = parameters[index];
        if (element.value.startsWith("!"))
            Negatives++;
    }
    //If we have less negatives then parameters - 1, then that means there are more positve thens 1
    if (Negatives < parameters.length - 1) {
        diagnoser.Add(offset, `Parameter: "${name}" can only have 1 positive test or/and multiple negatives test`, include_1.DiagnosticSeverity.error, "selector.attribute.test.postive_allnegatives");
    }
}
/**
 *
 * @param value
 * @param selector
 * @param diagnoser
 * @returns
 */
function selectorattribute_all(value, selector, diagnoser) {
    const parameters = selector.get(value.name);
    //Just check for duplicate tests
    if (parameters.length <= 1)
        return;
    const name = value.name;
    const avalue = value.value;
    const offset = value.offset;
    for (let I = 0; I < parameters.length; I++) {
        const first = parameters[I];
        if (first.value == avalue && first.offset !== offset)
            diagnoser.Add(offset + name.length + 1, `Duplicate test statement found for: "${name}"`, include_1.DiagnosticSeverity.warning, "selector.attribute.test.duplilcate");
    }
}
//# sourceMappingURL=Selector.js.map