"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Diagnose = void 0;
const bc_minecraft_bedrock_project_1 = require("bc-minecraft-bedrock-project");
const Json_1 = require("../../Json/Json");
const diagnostics_1 = require("../Animation Controllers/diagnostics");
const anim_or_controller_1 = require("../anim or controller");
const bc_minecraft_molang_1 = require("bc-minecraft-molang");
const bc_minecraft_bedrock_types_1 = require("bc-minecraft-bedrock-types");
const diagnostics_2 = require("../../Molang/diagnostics");
const diagnostics_3 = require("../Render Controller/diagnostics");
const diagnose_1 = require("../Model/diagnose");
const entry_1 = require("../Texture Atlas/entry");
const diagnose_2 = require("../Particle/diagnose");
const Script_1 = require("../../Minecraft/Script");
const diagnostics_4 = require("../Sounds/diagnostics");
/**Diagnoses the given document as an RP entity
 * @param doc The text document to diagnose
 * @param diagnoser The diagnoser builder to receive the errors*/
function Diagnose(doc, diagnoser) {
    //No behaviorpack check, entities can exist without their bp side (for servers)
    var _a, _b;
    //Check molang math functions
    (0, diagnostics_2.diagnose_molang)(doc.getText(), "entity", diagnoser);
    //Load entity
    const entity = Json_1.Json.LoadReport(doc, diagnoser);
    if (!bc_minecraft_bedrock_project_1.Internal.ResourcePack.Entity.is(entity))
        return;
    const container = entity["minecraft:client_entity"].description;
    const entityGathered = bc_minecraft_bedrock_project_1.ResourcePack.Entity.Process(doc);
    if (!entityGathered)
        return;
    if (!entityGathered.molang)
        entityGathered.molang = bc_minecraft_molang_1.Molang.MolangFullSet.harvest(doc.getText());
    //Check animations / animation controllers
    bc_minecraft_bedrock_types_1.Types.Definition.forEach(container.animations, (reference, anim_id) => (0, anim_or_controller_1.animation_or_controller_diagnose_implementation)(anim_id, entityGathered, "entity", diagnoser));
    //Check animation controllers
    (_a = container.animation_controllers) === null || _a === void 0 ? void 0 : _a.forEach((controller) => {
        const temp = flatten(controller);
        if (temp)
            (0, diagnostics_1.animation_controller_diagnose_implementation)(temp, entityGathered, "entity", diagnoser);
    });
    //Check render controllers
    (_b = container.render_controllers) === null || _b === void 0 ? void 0 : _b.forEach((controller) => {
        const temp = getkey(controller);
        if (temp)
            (0, diagnostics_3.render_controller_diagnose_implementation)(temp, entityGathered, "entity", diagnoser);
    });
    //Check models
    bc_minecraft_bedrock_types_1.Types.Definition.forEach(container.geometry, (ref, modelid) => (0, diagnose_1.resourcepack_has_model)(modelid, entityGathered.id, diagnoser));
    //check particles
    bc_minecraft_bedrock_types_1.Types.Definition.forEach(container.particle_effects, (ref, part_id) => (0, diagnose_2.resourcepack_particle_diagnose)(part_id, diagnoser));
    //Get pack
    const pack = diagnoser.context.getCache().ResourcePacks.get(doc.uri);
    if (pack === undefined)
        return;
    const rp_files = diagnoser.context
        .getFiles(pack.folder, ["**/textures/**/*.{tga,png,jpg,jpeg}"], pack.context.ignores)
        .map((item) => item.replace(/\\/gi, "/"));
    //Check if entity has textures defined
    bc_minecraft_bedrock_types_1.Types.Definition.forEach(container.textures, (ref, id) => {
        (0, entry_1.texture_files_diagnose)(container.identifier, id, rp_files, diagnoser);
    });
    //Check if entity has sounds defined
    (0, diagnostics_4.diagnose_resourcepack_sounds)(container.sound_effects, diagnoser);
    //Script check
    if (container.scripts)
        (0, Script_1.diagnose_script)(diagnoser, container.scripts, container.animations);
}
exports.Diagnose = Diagnose;
function flatten(data) {
    if (typeof data === "string")
        return data;
    const key = Object.getOwnPropertyNames(data)[0];
    if (key)
        return data[key];
    return undefined;
}
function getkey(data) {
    if (typeof data === "string")
        return data;
    return Object.getOwnPropertyNames(data)[0];
}
//# sourceMappingURL=entry.js.map