"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.sound_files_diagnose = exports.Diagnose = void 0;
const bc_minecraft_bedrock_project_1 = require("bc-minecraft-bedrock-project");
const bc_minecraft_bedrock_vanilla_data_1 = require("bc-minecraft-bedrock-vanilla-data");
const main_1 = require("../../../../main");
const Definitions_1 = require("../../Definitions");
const Json_1 = require("../../Json/Json");
/**Diagnoses the given document as a `sound_definitions` file
 * @param doc The text document to diagnose
 * @param diagnoser The diagnoser builder to receive the errors*/
function Diagnose(doc, diagnoser) {
    const defintions = Json_1.Json.LoadReport(doc, diagnoser);
    if (!bc_minecraft_bedrock_project_1.Internal.ResourcePack.SoundDefinitions.is(defintions))
        return;
    //Get pack for files search
    const pack = diagnoser.context.getCache().ResourcePacks.get(doc.uri);
    if (pack === undefined)
        return;
    const sounds = defintions.sound_definitions;
    const sound_files = diagnoser.context
        .getFiles(pack.folder, ["**/sounds/**/*.{fsb,wav,ogg}"], pack.context.ignores)
        .map((item) => item.replace(/\\/gi, "/"));
    //For each sound definition
    bc_minecraft_bedrock_project_1.Map.forEach(sounds, (sound, sound_id) => {
        //For each file reference
        sound.sounds.forEach((soundSpec) => {
            if (typeof soundSpec === "string") {
                sound_files_diagnose(sound_id, soundSpec, sound_files, diagnoser);
            }
            else {
                const name = soundSpec.name;
                if (typeof name === "string") {
                    sound_files_diagnose(sound_id, name, sound_files, diagnoser);
                }
            }
        });
    });
}
exports.Diagnose = Diagnose;
function sound_files_diagnose(owner, file, files, diagnoser) {
    for (let I = 0; I < files.length; I++) {
        if (files[I].includes(file)) {
            //Found then return out
            return;
        }
    }
    if (bc_minecraft_bedrock_vanilla_data_1.MinecraftData.ResourcePack.hasSound(file, (0, Definitions_1.education_enabled)(diagnoser)))
        return;
    diagnoser.Add(`${owner}/${file}`, `Cannot find file: ${file}`, main_1.DiagnosticSeverity.error, "resourpack.sound.missing");
}
exports.sound_files_diagnose = sound_files_diagnose;
//# sourceMappingURL=entry.js.map