"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.texture_files_diagnose = exports.DiagnoseFlipbook = exports.DiagnoseAtlas = void 0;
const bc_minecraft_bedrock_project_1 = require("bc-minecraft-bedrock-project");
const bc_minecraft_bedrock_vanilla_data_1 = require("bc-minecraft-bedrock-vanilla-data");
const Severity_1 = require("../../../Types/DiagnosticsBuilder/Severity");
const Definitions_1 = require("../../Definitions");
const Json_1 = require("../../Json/Json");
/**Diagnoses the given document as a texture atlas
 * @param doc The text document to diagnose
 * @param diagnoser The diagnoser builder to receive the errors*/
function DiagnoseAtlas(doc, diagnoser) {
    const defintions = Json_1.Json.LoadReport(doc, diagnoser);
    if (!TextureAtlas.is(defintions))
        return;
    //Get pack for files search
    const pack = diagnoser.context.getCache().ResourcePacks.get(doc.uri);
    if (pack === undefined)
        return;
    const texture_data = defintions.texture_data;
    const texture_files = diagnoser.context
        .getFiles(pack.folder, ["**/textures/**/*.{tga,png,jpg,jpeg}"], pack.context.ignores)
        .map((item) => item.replace(/\\/gi, "/"));
    //Check if files exists
    const check_file_spec = (texture_id, item) => {
        if (typeof item.path === "string") {
            texture_files_diagnose(texture_id, item.path, texture_files, diagnoser);
        }
        if (item.variations) {
            item.variations.forEach((subitem) => {
                if (typeof subitem.path === "string") {
                    texture_files_diagnose(texture_id, subitem.path, texture_files, diagnoser);
                }
            });
        }
    };
    const check_file_string = (texture_id, item) => {
        texture_files_diagnose(texture_id, item, texture_files, diagnoser);
    };
    bc_minecraft_bedrock_project_1.Map.forEach(texture_data, (data, texture_id) => {
        //If texture
        if (typeof data.textures === "string") {
            check_file_string(texture_id, data.textures);
            //If array of items
        }
        else if (Array.isArray(data.textures)) {
            data.textures.forEach((texture) => {
                if (typeof texture === "string") {
                    texture_files_diagnose(texture_id, texture, texture_files, diagnoser);
                }
                else {
                    check_file_spec(texture_id, texture);
                }
            });
        }
        else {
            check_file_spec(texture_id, data.textures);
        }
    });
}
exports.DiagnoseAtlas = DiagnoseAtlas;
/**Diagnoses the given document as a texture flipbook file
 * @param doc The text document to diagnose
 * @param diagnoser The diagnoser builder to receive the errors*/
function DiagnoseFlipbook(doc, diagnoser) {
    //TODO add rp diagnostics
}
exports.DiagnoseFlipbook = DiagnoseFlipbook;
function texture_files_diagnose(owner, file, files, diagnoser) {
    for (let I = 0; I < files.length; I++) {
        if (files[I].includes(file)) {
            //Found then return
            return;
        }
    }
    if (bc_minecraft_bedrock_vanilla_data_1.MinecraftData.ResourcePack.hasTexture(file, (0, Definitions_1.education_enabled)(diagnoser)))
        return;
    diagnoser.Add(`${owner}/${file}`, `Cannot find file: ${file}`, Severity_1.DiagnosticSeverity.error, "resourpack.texture.missing");
}
exports.texture_files_diagnose = texture_files_diagnose;
var TextureAtlas;
(function (TextureAtlas) {
    function is(value) {
        if (typeof value === "object" && typeof value.texture_data === "object")
            return true;
        return false;
    }
    TextureAtlas.is = is;
})(TextureAtlas || (TextureAtlas = {}));
//# sourceMappingURL=entry.js.map