"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.is_animation_or_controller = exports.anim_or_contr = exports.animation_reference_diagnose = exports.animation_or_controller_diagnose = exports.animation_or_controller_diagnose_implementation = void 0;
const bc_minecraft_bedrock_vanilla_data_1 = require("bc-minecraft-bedrock-vanilla-data");
const Severity_1 = require("../../Types/DiagnosticsBuilder/Severity");
const Definitions_1 = require("../Definitions");
const diagnostics_1 = require("./Animation Controllers/diagnostics");
const diagnostics_2 = require("./Animation/diagnostics");
function animation_or_controller_diagnose_implementation(id, user, ownerType, diagnoser) {
    switch (is_animation_or_controller(id, diagnoser)) {
        case anim_or_contr.animation:
            return (0, diagnostics_2.animation_diagnose_implementation)(id, user, ownerType, diagnoser);
        case anim_or_contr.controller:
            return (0, diagnostics_1.animation_controller_diagnose_implementation)(id, user, ownerType, diagnoser);
        case anim_or_contr.neither:
            diagnoser.Add(id, `Cannot find animation / animation controller: ${id}`, Severity_1.DiagnosticSeverity.error, "resourcepack.anim_or_controller.missing");
    }
}
exports.animation_or_controller_diagnose_implementation = animation_or_controller_diagnose_implementation;
function animation_or_controller_diagnose(id, diagnoser) {
    switch (is_animation_or_controller(id.text, diagnoser)) {
        case anim_or_contr.controller:
        case anim_or_contr.animation:
            return;
        case anim_or_contr.neither:
            diagnoser.Add(id, `Cannot find animation / animation controller: ${id}`, Severity_1.DiagnosticSeverity.error, "mcfunction.anim_or_controller.missing");
            return;
    }
}
exports.animation_or_controller_diagnose = animation_or_controller_diagnose;
function animation_reference_diagnose(value, diagnoser) {
    const data = diagnoser.context.getCache();
    const id = value.text;
    let out = false;
    //Project?
    data.ResourcePacks.entities.forEach(entity => {
        if (entity.animations.defined.includes(id))
            out = true;
    });
    if (out)
        return;
    //Vanilla?
    bc_minecraft_bedrock_vanilla_data_1.MinecraftData.vanilla.ResourcePack.entities.forEach(entity => {
        if (entity.animations.includes(id))
            out = true;
    });
    if (out)
        return;
    diagnoser.Add(value, `Cannot find animation / animation controller: ${value.text}`, Severity_1.DiagnosticSeverity.error, "mcfunction.anim_or_controller.missing");
}
exports.animation_reference_diagnose = animation_reference_diagnose;
var anim_or_contr;
(function (anim_or_contr) {
    anim_or_contr[anim_or_contr["animation"] = 0] = "animation";
    anim_or_contr[anim_or_contr["controller"] = 1] = "controller";
    anim_or_contr[anim_or_contr["neither"] = 2] = "neither";
})(anim_or_contr = exports.anim_or_contr || (exports.anim_or_contr = {}));
/**
 *
 * @param id
 * @param diagnoser
 * @returns True if animation, false if controller
 */
function is_animation_or_controller(id, diagnoser) {
    const cache = diagnoser.context.getCache();
    if (cache.ResourcePacks.animations.has(id))
        return anim_or_contr.animation;
    if (cache.ResourcePacks.animation_controllers.has(id))
        return anim_or_contr.controller;
    const edu = (0, Definitions_1.education_enabled)(diagnoser);
    //Vanilla has render controller
    if (bc_minecraft_bedrock_vanilla_data_1.MinecraftData.ResourcePack.hasAnimation(id, edu))
        return anim_or_contr.animation;
    if (bc_minecraft_bedrock_vanilla_data_1.MinecraftData.ResourcePack.hasAnimationController(id, edu))
        return anim_or_contr.controller;
    return anim_or_contr.neither;
}
exports.is_animation_or_controller = is_animation_or_controller;
//# sourceMappingURL=anim%20or%20controller.js.map